@@ -1,10 +1,11 @@
 package edit
 
 import (
-	"github.com/elves/elvish/pkg/cli"
-	"github.com/elves/elvish/pkg/cli/addons/instant"
-	"github.com/elves/elvish/pkg/eval"
-	"github.com/elves/elvish/pkg/parse"
+	"src.elv.sh/pkg/cli"
+	"src.elv.sh/pkg/cli/mode"
+	"src.elv.sh/pkg/cli/tk"
+	"src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/parse"
 )
 
 //elvdoc:var -instant:binding
@@ -21,17 +22,17 @@ import (
 // Elvish will attempt to evaluate `sudo rm -rf /` when you typed that far.
 
 func initInstant(ed *Editor, ev *eval.Evaler, nb eval.NsBuilder) {
-	bindingVar := newBindingVar(EmptyBindingMap)
-	binding := newMapBinding(ed, ev, bindingVar)
+	bindingVar := newBindingVar(emptyBindingsMap)
+	bindings := newMapBindings(ed, ev, bindingVar)
 	nb.AddNs("-instant",
 		eval.NsBuilder{
 			"binding": bindingVar,
 		}.AddGoFns("<edit:-instant>:", map[string]interface{}{
-			"start": func() { instantStart(ed.app, ev, binding) },
+			"start": func() { instantStart(ed.app, ev, bindings) },
 		}).Ns())
 }
 
-func instantStart(app cli.App, ev *eval.Evaler, binding cli.Handler) {
+func instantStart(app cli.App, ev *eval.Evaler, bindings tk.Bindings) {
 	execute := func(code string) ([]string, error) {
 		outPort, collect, err := eval.StringCapturePort()
 		if err != nil {
@@ -44,5 +45,13 @@ func instantStart(app cli.App, ev *eval.Evaler, binding cli.Handler) {
 				Interrupt: eval.ListenInterrupts})
 		return collect(), err
 	}
-	instant.Start(app, instant.Config{Binding: binding, Execute: execute})
+	w, err := mode.NewInstant(app,
+		mode.InstantSpec{Bindings: bindings, Execute: execute})
+	if w != nil {
+		app.SetAddon(w, false)
+		app.Redraw()
+	}
+	if err != nil {
+		app.Notify(err.Error())
+	}
 }
