@@ -1,16 +1,21 @@
 package eval_test
 
 import (
+	"os"
 	"testing"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
-	"github.com/elves/elvish/pkg/eval/vals"
+	"src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/eval/errs"
+	. "src.elv.sh/pkg/eval/evaltest"
+	"src.elv.sh/pkg/eval/vals"
 )
 
 func TestPut(t *testing.T) {
 	Test(t,
 		That(`put foo bar`).Puts("foo", "bar"),
 		That(`put $nil`).Puts(nil),
+
+		thatOutputErrorIsBubbled("put foo"),
 	)
 }
 
@@ -21,6 +26,10 @@ func TestReadUpto(t *testing.T) {
 		That("print abcd | { read-upto c; slurp }").Puts("abc", "d"),
 		// read-upto reads up to EOF
 		That("print abcd | read-upto z").Puts("abcd"),
+		That("print abcd | read-upto cd").Throws(
+			errs.BadValue{What: "terminator",
+				Valid: "a single ASCII character", Actual: "cd"}),
+		thatOutputErrorIsBubbled("print abcd | read-upto c"),
 	)
 }
 
@@ -30,6 +39,7 @@ func TestReadLine(t *testing.T) {
 		That(`print "lf-ending\n" | read-line`).Puts("lf-ending"),
 		That(`print "crlf-ending\r\n" | read-line`).Puts("crlf-ending"),
 		That(`print "extra-cr\r\r\n" | read-line`).Puts("extra-cr\r"),
+		thatOutputErrorIsBubbled(`print eof-ending | read-line`),
 	)
 }
 
@@ -37,24 +47,28 @@ func TestPrint(t *testing.T) {
 	Test(t,
 		That(`print [foo bar]`).Prints("[foo bar]"),
 		That(`print foo bar &sep=,`).Prints("foo,bar"),
+		thatOutputErrorIsBubbled("print foo"),
 	)
 }
 
 func TestEcho(t *testing.T) {
 	Test(t,
 		That(`echo [foo bar]`).Prints("[foo bar]\n"),
+		thatOutputErrorIsBubbled("echo foo"),
 	)
 }
 
 func TestPprint(t *testing.T) {
 	Test(t,
 		That(`pprint [foo bar]`).Prints("[\n foo\n bar\n]\n"),
+		thatOutputErrorIsBubbled("pprint foo"),
 	)
 }
 
 func TestReprCmd(t *testing.T) {
 	Test(t,
 		That(`repr foo bar ['foo bar']`).Prints("foo bar ['foo bar']\n"),
+		thatOutputErrorIsBubbled("repr foo"),
 	)
 }
 
@@ -62,21 +76,32 @@ func TestShow(t *testing.T) {
 	Test(t,
 		// A sanity test that show writes something.
 		That(`show ?(fail foo) | !=s (slurp) ''`).Puts(true),
+		thatOutputErrorIsBubbled("repr ?(fail foo)"),
 	)
 }
 
 func TestOnlyBytesAndOnlyValues(t *testing.T) {
 	Test(t,
-		// Baseline for only-{bytes,values}
+		// Baseline
 		That(`{ print bytes; put values }`).Prints("bytes").Puts("values"),
 		That(`{ print bytes; put values } | only-bytes`).Prints("bytes").Puts(),
+		thatOutputErrorIsBubbled("{ print bytes; put values } | only-bytes"),
+	)
+}
+
+func TestOnlyValues(t *testing.T) {
+	Test(t,
+		// Baseline
+		That(`{ print bytes; put values }`).Prints("bytes").Puts("values"),
 		That(`{ print bytes; put values } | only-values`).Prints("").Puts("values"),
+		thatOutputErrorIsBubbled("{ print bytes; put values } | only-values"),
 	)
 }
 
 func TestSlurp(t *testing.T) {
 	Test(t,
 		That(`print "a\nb" | slurp`).Puts("a\nb"),
+		thatOutputErrorIsBubbled(`print "a\nb" | slurp`),
 	)
 }
 
@@ -84,6 +109,19 @@ func TestFromLines(t *testing.T) {
 	Test(t,
 		That(`print "a\nb" | from-lines`).Puts("a", "b"),
 		That(`print "a\nb\n" | from-lines`).Puts("a", "b"),
+		thatOutputErrorIsBubbled(`print "a\nb\n" | from-lines`),
+	)
+}
+
+func TestFromTerminated(t *testing.T) {
+	Test(t,
+		That(`print "a\nb\x00\x00c\x00d" | from-terminated "\x00"`).Puts("a\nb", "", "c", "d"),
+		That(`print "a\x00b\x00" | from-terminated "\x00"`).Puts("a", "b"),
+		That(`print aXbXcXXd | from-terminated "X"`).Puts("a", "b", "c", "", "d"),
+		That(`from-terminated "xyz"`).Throws(
+			errs.BadValue{What: "terminator",
+				Valid: "a single ASCII character", Actual: "xyz"}),
+		thatOutputErrorIsBubbled("print aXbX | from-terminated X"),
 	)
 }
 
@@ -95,12 +133,25 @@ func TestFromJson(t *testing.T) {
 		That(`echo '[null, "foo"]' | from-json`).Puts(
 			vals.MakeList(nil, "foo")),
 		That(`echo 'invalid' | from-json`).Throws(AnyError),
+		thatOutputErrorIsBubbled(`echo '[]' | from-json`),
 	)
 }
 
 func TestToLines(t *testing.T) {
 	Test(t,
 		That(`put "l\norem" ipsum | to-lines`).Prints("l\norem\nipsum\n"),
+		thatOutputErrorIsBubbled("to-lines [foo]"),
+	)
+}
+
+func TestToTerminated(t *testing.T) {
+	Test(t,
+		That(`put "l\norem" ipsum | to-terminated "\x00"`).Prints("l\norem\x00ipsum\x00"),
+		That(`to-terminated "X" [a b c]`).Prints("aXbXcX"),
+		That(`to-terminated "XYZ" [a b c]`).Throws(
+			errs.BadValue{What: "terminator",
+				Valid: "a single ASCII character", Actual: "XYZ"}),
+		thatOutputErrorIsBubbled(`to-terminated "X" [a b c]`),
 	)
 }
 
@@ -111,6 +162,7 @@ func TestToJson(t *testing.T) {
 "foo"
 `),
 		That(`put [$nil foo] | to-json`).Prints("[null,\"foo\"]\n"),
+		thatOutputErrorIsBubbled("to-json [foo]"),
 	)
 }
 
@@ -129,9 +181,9 @@ func TestPrintf(t *testing.T) {
 		That(`printf '%t' $true`).Prints("true"),
 		That(`printf '%t' $nil`).Prints("false"),
 
-		That(`printf '%3d' (float64 5)`).Prints("  5"),
+		That(`printf '%3d' (num 5)`).Prints("  5"),
 		That(`printf '%3d' 5`).Prints("  5"),
-		That(`printf '%08b' (float64 5)`).Prints("00000101"),
+		That(`printf '%08b' (num 5)`).Prints("00000101"),
 		That(`printf '%08b' 5`).Prints("00000101"),
 
 		That(`printf '%.1f' 3.1415`).Prints("3.1"),
@@ -148,5 +200,11 @@ func TestPrintf(t *testing.T) {
 		That(`printf '%d' 3.5`).Prints("%!d(cannot parse as integer: 3.5)"),
 		// Unsupported verb
 		That(`printf '%A' foo`).Prints("%!A(unsupported formatting verb)"),
+
+		thatOutputErrorIsBubbled("printf foo"),
 	)
 }
+
+func thatOutputErrorIsBubbled(code string) TestCase {
+	return That(code + " >&-").Throws(OneOfErrors(os.ErrInvalid, eval.ErrNoValueOutput))
+}
