@@ -4,17 +4,16 @@ import (
 	"os"
 	"testing"
 
-	"github.com/elves/elvish/pkg/env"
-	. "github.com/elves/elvish/pkg/eval"
+	"src.elv.sh/pkg/env"
+	. "src.elv.sh/pkg/eval"
 
-	. "github.com/elves/elvish/pkg/eval/evaltest"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/elves/elvish/pkg/testutil"
+	. "src.elv.sh/pkg/eval/evaltest"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/testutil"
 )
 
 func TestChdir(t *testing.T) {
-	dst, cleanup := testutil.TestDir()
-	defer cleanup()
+	dst := testutil.TempDir(t)
 
 	ev := NewEvaler()
 
@@ -45,8 +44,7 @@ func TestChdir(t *testing.T) {
 }
 
 func TestChdirElvishHooks(t *testing.T) {
-	dst, cleanup := testutil.TestDir()
-	defer cleanup()
+	dst := testutil.TempDir(t)
 
 	back := saveWd()
 	defer back()
@@ -63,8 +61,7 @@ func TestChdirElvishHooks(t *testing.T) {
 }
 
 func TestChdirError(t *testing.T) {
-	_, cleanup := testutil.InTestDir()
-	defer cleanup()
+	testutil.InTempDir(t)
 
 	ev := NewEvaler()
 	err := ev.Chdir("i/dont/exist")
