@@ -9,14 +9,14 @@ import (
 	"strconv"
 	"sync"
 
-	"github.com/elves/elvish/pkg/daemon"
-	"github.com/elves/elvish/pkg/diag"
-	"github.com/elves/elvish/pkg/env"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/eval/vars"
-	"github.com/elves/elvish/pkg/logutil"
-	"github.com/elves/elvish/pkg/parse"
-	"github.com/xiaq/persistent/vector"
+	"src.elv.sh/pkg/daemon/daemondefs"
+	"src.elv.sh/pkg/diag"
+	"src.elv.sh/pkg/env"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/eval/vars"
+	"src.elv.sh/pkg/logutil"
+	"src.elv.sh/pkg/parse"
+	"src.elv.sh/pkg/persistent/vector"
 )
 
 var logger = logutil.GetLogger("[eval] ")
@@ -56,8 +56,10 @@ type Evaler struct {
 
 	// State of the module system.
 	//
-	// Library directory.
-	libDir string
+	// Directories to search libraries.
+	libDirs []string
+	// Default directory to install libraries.
+	libInstallDir string
 	// Internal modules are indexed by use specs. External modules are indexed by
 	// absolute paths.
 	modules map[string]*Ns
@@ -80,14 +82,13 @@ type Evaler struct {
 	// Dependencies.
 	//
 	// TODO: Remove these dependency by providing more general extension points.
-	daemonClient daemon.Client
-	editor       Editor
+	daemonClient daemondefs.Client
 }
 
 // Editor is the interface that the line editor has to satisfy. It is needed so
 // that this package does not depend on the edit package.
 type Editor interface {
-	Notify(string, ...interface{})
+	RunAfterCommandHooks(src parse.Source, duration float64, err error)
 }
 
 //elvdoc:var after-chdir
@@ -248,17 +249,23 @@ func (ev *Evaler) registerDeprecation(d deprecation) bool {
 }
 
 // Returns libdir.
-func (ev *Evaler) getLibDir() string {
+func (ev *Evaler) getLibDirs() []string {
 	ev.mu.RLock()
 	defer ev.mu.RUnlock()
-	return ev.libDir
+	return ev.libDirs
 }
 
 // SetLibDir sets the library directory for finding external modules.
-func (ev *Evaler) SetLibDir(libDir string) {
+func (ev *Evaler) SetLibDirs(libDirs []string) {
 	ev.mu.Lock()
 	defer ev.mu.Unlock()
-	ev.libDir = libDir
+	ev.libDirs = libDirs
+}
+
+func (ev *Evaler) SetLibInstallDir(libInstallDir string) {
+	ev.mu.Lock()
+	defer ev.mu.Unlock()
+	ev.libInstallDir = libInstallDir
 }
 
 // AddModule add an internal module so that it can be used with "use $name" from
@@ -333,26 +340,19 @@ func (ev *Evaler) AddAfterChdir(f func(string)) {
 }
 
 // SetDaemonClient sets the daemon client associated with the Evaler.
-func (ev *Evaler) SetDaemonClient(client daemon.Client) {
+func (ev *Evaler) SetDaemonClient(client daemondefs.Client) {
 	ev.mu.Lock()
 	defer ev.mu.Unlock()
 	ev.daemonClient = client
 }
 
 // DaemonClient returns the daemon client associated with the Evaler.
-func (ev *Evaler) DaemonClient() daemon.Client {
+func (ev *Evaler) DaemonClient() daemondefs.Client {
 	ev.mu.RLock()
 	defer ev.mu.RUnlock()
 	return ev.daemonClient
 }
 
-// DaemonClient returns the editor associated with the Evaler.
-func (ev *Evaler) Editor() Editor {
-	ev.mu.RLock()
-	defer ev.mu.RUnlock()
-	return ev.editor
-}
-
 // Chdir changes the current directory. On success it also updates the PWD
 // environment variable and records the new directory in the directory history.
 // It runs the functions in beforeChdir immediately before changing the
@@ -422,7 +422,7 @@ func (ev *Evaler) Eval(src parse.Source, cfg EvalCfg) error {
 	cfg.fillDefaults()
 	errFile := cfg.Ports[2].File
 
-	tree, err := parse.ParseWithDeprecation(src, errFile)
+	tree, err := parse.Parse(src, parse.Config{WarningWriter: errFile})
 	if err != nil {
 		return err
 	}
@@ -498,7 +498,9 @@ func (ev *Evaler) prepareFrame(src parse.Source, cfg EvalCfg) (*Frame, func()) {
 		intCh, intChCleanup = cfg.Interrupt()
 	}
 
-	fm := &Frame{ev, src, cfg.Global, new(Ns), intCh, cfg.Ports, nil, false}
+	ports := fillDefaultDummyPorts(cfg.Ports)
+
+	fm := &Frame{ev, src, cfg.Global, new(Ns), intCh, ports, nil, false}
 	return fm, func() {
 		if intChCleanup != nil {
 			intChCleanup()
@@ -506,19 +508,33 @@ func (ev *Evaler) prepareFrame(src parse.Source, cfg EvalCfg) (*Frame, func()) {
 		if cfg.PutInFg {
 			err := putSelfInFg()
 			if err != nil {
-				fmt.Fprintln(cfg.Ports[2].File,
+				fmt.Fprintln(ports[2].File,
 					"failed to put myself in foreground:", err)
 			}
 		}
 	}
 }
 
+func fillDefaultDummyPorts(ports []*Port) []*Port {
+	growPorts(&ports, 3)
+	if ports[0] == nil {
+		ports[0] = DummyInputPort
+	}
+	if ports[1] == nil {
+		ports[1] = DummyOutputPort
+	}
+	if ports[2] == nil {
+		ports[2] = DummyOutputPort
+	}
+	return ports
+}
+
 // Check checks the given source code for any parse error and compilation error.
 // It always tries to compile the code even if there is a parse error; both
 // return values may be non-nil. If w is not nil, deprecation messages are
 // written to it.
 func (ev *Evaler) Check(src parse.Source, w io.Writer) (*parse.Error, *diag.Error) {
-	tree, parseErr := parse.ParseWithDeprecation(src, w)
+	tree, parseErr := parse.Parse(src, parse.Config{WarningWriter: w})
 	return parse.GetError(parseErr), ev.CheckTree(tree, w)
 }
 
