@@ -7,9 +7,9 @@ import (
 	"strings"
 	"unicode"
 
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/glob"
-	"github.com/elves/elvish/pkg/parse"
+	"src.elv.sh/pkg/eval/vals"
+	"src.elv.sh/pkg/glob"
+	"src.elv.sh/pkg/parse"
 )
 
 // An ephemeral value generated when evaluating tilde and wildcards.
